@@ -29,7 +29,7 @@
 #define TSMUX_PCR_OFFSET (TSMUX_CLOCK_FREQ / 8)
 
 /* Times per second to write PCR */
-#define TSMUX_DEFAULT_PCR_FREQ (10)
+#define TSMUX_DEFAULT_PCR_FREQ (25)
 
 /* PAT frequency (1/10th sec) */
 #define TSMUX_DEFAULT_PAT_FREQ (TSMUX_CLOCK_FREQ / 10)
@@ -365,7 +365,7 @@ tsmux_packet_out (TsMux * mux)
     return TRUE;
 
   return mux->write_func (mux->packet_buf, TSMUX_PACKET_LENGTH,
-      mux->write_func_data);
+      mux->write_func_data, mux->new_pcr);
 }
 
 /*
@@ -460,7 +460,7 @@ tsmux_write_adaptation_field (guint8 * buf,
       pcr_ext = (pi->pcr % 300);
 
       flags |= 0x10;
-      TS_DEBUG ("Writing PCR");
+      TS_DEBUG ("Writing PCR %" G_GUINT64_FORMAT " + ext %u", pcr_base, pcr_ext);
       buf[pos++] = (pcr_base >> 25) & 0xff;
       buf[pos++] = (pcr_base >> 17) & 0xff;
       buf[pos++] = (pcr_base >> 9) & 0xff;
@@ -593,14 +593,19 @@ tsmux_write_ts_header (guint8 * buf, TsMuxPacketInfo * pi,
     /* Flag the presence of a payload */
     adaptation_flag |= 0x10;
 
-    /* Write the byte out */
-    buf[3] = adaptation_flag;
-
     /* We must have enough data to fill the payload, or some calculation
      * went wrong */
     g_assert (payload_len <= pi->stream_avail);
+
+    /* Packet with payload, increment the continuity counter */
+    pi->packet_count++;
   }
 
+  /* Write the byte of transport_scrambling_control, adaptation_field_control 
+   * + continuity counter out */
+  buf[3] = adaptation_flag;
+
+
   if (write_adapt) {
     TS_DEBUG ("Adaptation field of size >= %d + %d bytes payload",
         adapt_len, payload_len);
@@ -608,8 +613,6 @@ tsmux_write_ts_header (guint8 * buf, TsMuxPacketInfo * pi,
     TS_DEBUG ("Payload of %d bytes only", payload_len);
   }
 
-  pi->packet_count++;
-
   return TRUE;
 }
 
@@ -629,6 +632,8 @@ tsmux_write_stream_packet (TsMux * mux, TsMuxStream * stream)
   TsMuxPacketInfo *pi = &stream->pi;
   gboolean res;
 
+  
+  mux->new_pcr = -1;
   g_return_val_if_fail (mux != NULL, FALSE);
   g_return_val_if_fail (stream != NULL, FALSE);
 
@@ -638,10 +643,15 @@ tsmux_write_stream_packet (TsMux * mux, TsMuxStream * stream)
     gboolean write_pat;
     GList *cur;
 
+    if (cur_pts != -1) {
+      TS_DEBUG ("TS for PCR stream is %" G_GINT64_FORMAT, cur_pts);
+    }
+
     /* FIXME: The current PCR needs more careful calculation than just
      * writing a fixed offset */
     if (cur_pts != -1 && (cur_pts >= TSMUX_PCR_OFFSET))
-      cur_pcr = cur_pts - TSMUX_PCR_OFFSET;
+      cur_pcr = (cur_pts - TSMUX_PCR_OFFSET) * 
+                     (TSMUX_SYS_CLOCK_FREQ / TSMUX_CLOCK_FREQ);
 
     /* Need to decide whether to write a new PCR in this packet */
     if (stream->last_pcr == -1 ||
@@ -652,18 +662,19 @@ tsmux_write_stream_packet (TsMux * mux, TsMuxStream * stream)
           TSMUX_PACKET_FLAG_ADAPTATION | TSMUX_PACKET_FLAG_WRITE_PCR;
       stream->pi.pcr = cur_pcr;
       stream->last_pcr = cur_pcr;
+      mux->new_pcr = cur_pcr;
     }
 
     /* check if we need to rewrite pat */
     if (mux->last_pat_ts == -1 || mux->pat_changed)
       write_pat = TRUE;
-    else if (cur_pts != -1 && cur_pts >= mux->last_pat_ts + mux->pat_frequency)
+    else if (cur_pcr >= mux->last_pat_ts + mux->pat_frequency)
       write_pat = TRUE;
     else
       write_pat = FALSE;
 
     if (write_pat) {
-      mux->last_pat_ts = cur_pts;
+      mux->last_pat_ts = cur_pcr;
       if (!tsmux_write_pat (mux))
         return FALSE;
     }
@@ -676,14 +687,13 @@ tsmux_write_stream_packet (TsMux * mux, TsMuxStream * stream)
 
       if (program->last_pmt_ts == -1 || program->pmt_changed)
         write_pmt = TRUE;
-      else if (cur_pts != -1
-          && cur_pts >= program->last_pmt_ts + program->pmt_frequency)
+      else if (cur_pcr >= program->last_pmt_ts + program->pmt_frequency)
         write_pmt = TRUE;
       else
         write_pmt = FALSE;
 
       if (write_pmt) {
-        program->last_pmt_ts = cur_pts;
+        program->last_pmt_ts = cur_pcr;
         if (!tsmux_write_pmt (mux, program))
           return FALSE;
       }
@@ -772,8 +782,11 @@ tsmux_write_section (TsMux * mux, TsMuxSection * section)
     cur_in += payload_len;
     payload_remain -= payload_len;
 
-    if (G_UNLIKELY (!tsmux_packet_out (mux)))
+    if (G_UNLIKELY (!tsmux_packet_out (mux))){
+      mux->new_pcr = -1;
       return FALSE;
+    }
+    mux->new_pcr = -1;
   }
 
   return TRUE;
